==> org.eclipse.emf.cdo.server.db.source_3.0.1.v20110120-1046.jar: org/eclipse/emf/cdo/server/internal/db/DBStoreChunkReader.java <==
/**
 * Copyright (c) 2004 - 2010 Eike Stepper (Berlin, Germany) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Eike Stepper - initial API and implementation
 *    Stefan Winkler - Bug 283998: [DB] Chunk reading for multiple chunks fails
 *    Victor Roldan Betancort - Bug 283998: [DB] Chunk reading for multiple chunks fails
 */
package org.eclipse.emf.cdo.server.internal.db;

import org.eclipse.emf.cdo.common.revision.CDORevision;
import org.eclipse.emf.cdo.server.db.IDBStoreChunkReader;
import org.eclipse.emf.cdo.server.db.mapping.IClassMapping;
import org.eclipse.emf.cdo.server.db.mapping.IListMapping;
import org.eclipse.emf.cdo.server.db.mapping.IMappingStrategy;
import org.eclipse.emf.cdo.spi.server.StoreChunkReader;

import org.eclipse.emf.ecore.EStructuralFeature;

import java.util.List;

/**
 * @author Eike Stepper
 */
public class DBStoreChunkReader extends StoreChunkReader implements IDBStoreChunkReader
{
  private IListMapping referenceMapping;

  private StringBuilder builder = new StringBuilder();

  public DBStoreChunkReader(DBStoreAccessor accessor, CDORevision revision, EStructuralFeature feature)
  {
    super(accessor, revision, feature);
    IMappingStrategy mappingStrategy = accessor.getStore().getMappingStrategy();
    IClassMapping mapping = mappingStrategy.getClassMapping(revision.getEClass());
    referenceMapping = mapping.getListMapping(feature);
  }

  @Override
  public DBStoreAccessor getAccessor()
  {
    return (DBStoreAccessor)super.getAccessor();
  }

  @Override
  public void addSimpleChunk(int index)
  {
    super.addSimpleChunk(index);
    prepareAddition();

    builder.append(CDODBSchema.LIST_IDX);
    builder.append('=');
    builder.append(index);
  }

  @Override
  public void addRangedChunk(int fromIndex, int toIndex)
  {
    super.addRangedChunk(fromIndex, toIndex);
    prepareAddition();

    builder.append(CDODBSchema.LIST_IDX);
    builder.append(" BETWEEN "); //$NON-NLS-1$
    builder.append(fromIndex);
    builder.append(" AND "); //$NON-NLS-1$
    builder.append(toIndex - 1);
  }

  public List<Chunk> executeRead()
  {
    List<Chunk> chunks = getChunks();
    if (chunks.size() > 1)
    {
      builder.insert(0, '(');
      builder.append(')');
    }

    referenceMapping.readChunks(this, chunks, builder.toString());
    return chunks;
  }

  private void prepareAddition()
  {
    // If not empty, a chunk has been already added, and the next condition needs to be OR-ed
    if (builder.length() > 0)
    {
      builder.append(" OR "); //$NON-NLS-1$
    }
  }
}

==> org.eclipse.emf.cdo.server.db.source_4.0.1.v20111221-2220.jar: org/eclipse/emf/cdo/server/internal/db/DBStoreChunkReader.java <==
/**
 * Copyright (c) 2004 - 2011 Eike Stepper (Berlin, Germany) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Eike Stepper - initial API and implementation
 *    Stefan Winkler - Bug 283998: [DB] Chunk reading for multiple chunks fails
 *    Victor Roldan Betancort - Bug 283998: [DB] Chunk reading for multiple chunks fails
 */
package org.eclipse.emf.cdo.server.internal.db;

import org.eclipse.emf.cdo.common.revision.CDORevision;
import org.eclipse.emf.cdo.server.db.IDBStoreChunkReader;
import org.eclipse.emf.cdo.server.db.mapping.IClassMapping;
import org.eclipse.emf.cdo.server.db.mapping.IListMapping;
import org.eclipse.emf.cdo.server.db.mapping.IMappingStrategy;
import org.eclipse.emf.cdo.server.internal.db.mapping.horizontal.BasicAbstractListTableMapping;
import org.eclipse.emf.cdo.spi.server.StoreChunkReader;

import org.eclipse.emf.ecore.EStructuralFeature;

import java.util.List;

/**
 * @author Eike Stepper
 */
public class DBStoreChunkReader extends StoreChunkReader implements IDBStoreChunkReader
{
  private IListMapping referenceMapping;

  private StringBuilder builder = new StringBuilder();

  public DBStoreChunkReader(DBStoreAccessor accessor, CDORevision revision, EStructuralFeature feature)
  {
    super(accessor, revision, feature);
    IMappingStrategy mappingStrategy = accessor.getStore().getMappingStrategy();
    IClassMapping mapping = mappingStrategy.getClassMapping(revision.getEClass());
    referenceMapping = mapping.getListMapping(feature);
  }

  @Override
  public DBStoreAccessor getAccessor()
  {
    return (DBStoreAccessor)super.getAccessor();
  }

  @Override
  public void addSimpleChunk(int index)
  {
    super.addSimpleChunk(index);
    prepareAddition();

    if (referenceMapping instanceof BasicAbstractListTableMapping) // backport of Bug 366686; in 4.1 the new
                                                                   // IListMapping2 interface is used
    {
      ((BasicAbstractListTableMapping)referenceMapping).addSimpleChunkWhere(getAccessor(), getRevision().getID(),
          builder, index);
    }
    else
    {
      builder.append(CDODBSchema.LIST_IDX);
      builder.append('=');
      builder.append(index);
    }
  }

  @Override
  public void addRangedChunk(int fromIndex, int toIndex)
  {
    super.addRangedChunk(fromIndex, toIndex);
    prepareAddition();

    if (referenceMapping instanceof BasicAbstractListTableMapping)// backport of Bug 366686; in 4.1 the new
    // IListMapping2 interface is used
    {
      ((BasicAbstractListTableMapping)referenceMapping).addRangedChunkWhere(getAccessor(), getRevision().getID(),
          builder, fromIndex, toIndex);
    }
    else
    {
      builder.append(CDODBSchema.LIST_IDX);
      builder.append(" BETWEEN "); //$NON-NLS-1$
      builder.append(fromIndex);
      builder.append(" AND "); //$NON-NLS-1$
      builder.append(toIndex - 1);
    }
  }

  public List<Chunk> executeRead()
  {
    List<Chunk> chunks = getChunks();
    if (chunks.size() > 1)
    {
      builder.insert(0, '(');
      builder.append(')');
    }

    referenceMapping.readChunks(this, chunks, builder.toString());
    return chunks;
  }

  private void prepareAddition()
  {
    // If not empty, a chunk has been already added, and the next condition needs to be OR-ed
    if (builder.length() > 0)
    {
      builder.append(" OR "); //$NON-NLS-1$
    }
  }
}
